﻿using System;


namespace WS2812_USB_UART
{
    
    internal static class GammaAdjustment
    {

        static GammaAdjustment()
        {
            Helligkeit = 100;
        }


        private static int _Helligkeit;

        public static int Helligkeit
        {
            get { return _Helligkeit; }
            set
            {
                // Helligkeit grenzen 0 - 100
                if (value < 0) { value = 0; } else if (value > 100) { value = 100; }

                _Helligkeit = value;

                float helligkeit = value / 100f;

                CalcGammaTab(GammaKorTabR, 2.5f, helligkeit, 0.925f);
                CalcGammaTab(GammaKorTabG, 2.5f, helligkeit, 1.0f);
                CalcGammaTab(GammaKorTabB, 2.7f, helligkeit, 0.95f);
            }
        }


        private static byte[] GammaKorTabR = new byte[256];
        private static byte[] GammaKorTabG = new byte[256];
        private static byte[] GammaKorTabB = new byte[256];

        private static unsafe void CalcGammaTab(byte[] gammaTab, float gamma, float helligkeit, float faktor)
        {

            double f = helligkeit * faktor / 255.0;
            double g = gamma;

            fixed (byte* pGammaTab = gammaTab)
            {
                byte* pGT = pGammaTab;

                for (int i = 1; i < 256; i++)
                    { *++pGT = (byte)((Math.Pow(i * f, g) * 255) + 0.5); }
            }
        }



        public static unsafe void Adjust(byte[] rgbArrayDest, byte[] rgbAarrySource)
        {
            fixed (byte* pGammaTabR = GammaKorTabR, pGammaTabG = GammaKorTabG, pGammaTabB = GammaKorTabB,
                pRgbArrayDest = rgbArrayDest, pRgbArraySource = rgbAarrySource)
            {
                byte* pGTR = pGammaTabR;
                byte* pGTG = pGammaTabG;
                byte* pGTB = pGammaTabB;
                byte* pRgbAD = pRgbArrayDest;
                byte* pRgbAS = pRgbArraySource;

                for (int i = 0; i < rgbArrayDest.Length; i += 3)
                {
                    pRgbAD[i + 0] = pGTR[pRgbAS[i + 0]];
                    pRgbAD[i + 1] = pGTG[pRgbAS[i + 1]];
                    pRgbAD[i + 2] = pGTB[pRgbAS[i + 2]];
                }
            }
        }

    }


}
