﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.CompilerServices;


namespace WS2812_USB_UART
{
 
    internal struct RGBColor
    {
        public float R;
        public float G;
        public float B;

        public RGBColor(float R, float G, float B)
        {
            this.R = R;
            this.G = G;
            this.B = B;
        }

        public byte ByteR
        {
            get { return (byte)(R * 255); }
            set { R = value / 255f; }
        }

        public byte ByteG
        {
            get { return (byte)(G * 255); }
            set { G = value / 255f; }
        }
        
        public byte ByteB
        {
            get { return (byte)(B * 255); }
            set { B = value / 255f; }
        }
    }


    internal struct HSVColor
    {
        public float H; // 360°
        public float S; // 1.0
        public float V; // 1.0

        public HSVColor(float H, float S, float V)
        {
            this.H = H;
            this.S = S;
            this.V = V;
        }
    }


    internal static class HSV2RGB
    {
        // Converts an HSV color to an RGB color, according to the algorithm described at http://en.wikipedia.org/wiki/HSL_and_HSV
        [MethodImpl(MethodImplOptions.Synchronized)]
        public static RGBColor Convert(HSVColor HSV)
        {
            RGBColor RGB = new RGBColor();
            int Region;
            float Position;
            float p;
            float q;
            float t;

            Region = (int)((HSV.H / 60.0f) % 6);
            Position = (HSV.H % 60.0f) / 60.0f;
            p = HSV.V * (1.0f - HSV.S);
            q = HSV.V * (1.0f - (HSV.S * Position));
            t = HSV.V * (1.0f - (HSV.S * (1.0f - Position)));

            switch (Region)
            {
                case 0:
                    RGB.R = HSV.V;
                    RGB.G = t;
                    RGB.B = p;
                    break;

                case 1:
                    RGB.R = q;
                    RGB.G = HSV.V;
                    RGB.B = p;
                    break;

                case 2:
                    RGB.R = p;
                    RGB.G = HSV.V;
                    RGB.B = t;
                    break;

                case 3:
                    RGB.R = p;
                    RGB.G = q;
                    RGB.B = HSV.V;
                    break;

                case 4:
                    RGB.R = t;
                    RGB.G = p;
                    RGB.B = HSV.V;
                    break;

                case 5:
                    RGB.R = HSV.V;
                    RGB.G = p;
                    RGB.B = q;
                    break;
            }

            return RGB;
        }

    
    }
}
